<?php
/**
 * @name Server
 * @description Classe de base gérant les serveurs du plugin "Server"
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 07/11/2012
 */
include_once SNMP_CLASS;
include_once LIB_SNMP;
include_once DEVICE_CLASS;
class Server extends Device
{
	protected $memorySize;
	protected $cpuInfos;
	protected $processes;
	protected $hostSystemInfo;
	protected $storage;
	protected $interfaces;
	protected $ipv4addr;
	protected $tcpStats;
	protected $tcpTable;
	protected $udpStats;
	protected $udpTable;
	protected $ipStats;
	
	/*@NAME: __construct
	 *@IN: $dev un objet Device
	*@OUT: /
	*@TREAT: permet de construire un objet server à partir d'un objet "Device"
	*@EXCEPTION: 	/
	*/
	public function __construct($dev=null)
	{
		if(get_class($dev)=='Device')
		{
			parent::constructDevice($dev);
		}
		else
		{
			parent::__construct();
		}
	}
	/*@NAME: getCpuDescr
	 *@IN: $index=0 optionnel index du CPU
	*@OUT: Un tableau associatif ('index'=> index du cpu, 'description'=> description du cpu)
	*@TREAT: effectue des requêtes SNMP afin de retourner la description du processeur
	*	 Si index est égale à 0, la fonction effectuera un WALK et retournera la charge de tout les CPU dans le cas contraire, il retournera seulement la charge du
	*	processeur demandé
	*@EXCEPTION: 	DeviceSNMPException si une erreur SNMP survient
	*/
	public function getCpuDescr($index=0)
	{
		$cpu = array();
		if($index==0)
		{
			$l =parent::snmpwalkoid(".1.3.6.1.2.1.25.3.2.1.2");
			for (reset($l); $i = key($l); next($l))
			{
				if(preg_match('#HOST-RESOURCES-TYPES::hrDeviceProcessor|iso.3.6.1.2.1.25.3.1.3#',$l[$i]))
				{
					if(preg_match('#\.([0-9]*)$#',$i,$regs))
					{
						$load =parent::snmpget(".1.3.6.1.2.1.25.3.2.1.3.".$regs[1]);
						$cpu[]=array('index'=>$regs[1],'description'=>formatString($load));
					}
				}
			}
		}
		else
		{
			$load =parent::snmpget(".1.3.6.1.2.1.25.3.2.1.3.".$index);
			$cpu[]=array('index'=>$index,'description'=>formatString($load));
		}
		return $cpu;
	}
	/*@NAME: getCpuLoad
	 *@IN: $index=0 optionnel index du CPU
	*@OUT: Un tableau associatif ('index'=> index du cpu, 'load'=> charge du CPU)
	*@TREAT: effectue des requêtes SNMP afin de retourner la charge CPU du serveur de la dernière minute
	*	 Si index est égale à 0, la fonction effectuera un WALK et retournera la charge de tout les CPU dans le cas contraire, il retournera seulement la charge du
	*	processeur demandé
	*@EXCEPTION: 	DeviceSNMPException si une erreur SNMP survient
	*/
	public function getCpuLoad($index=0)
	{
		$cpu = array();
		if($index==0)
		{
			$l =parent::snmpwalkoid(".1.3.6.1.2.1.25.3.2.1.2");
			for (reset($l); $i = key($l); next($l))
			{
				if(preg_match('#HOST-RESOURCES-TYPES::hrDeviceProcessor|iso.3.6.1.2.1.25.3.1.3#',$l[$i]))
				{
					if(preg_match('#\.([0-9]*)$#',$i,$regs))
					{
						$load =parent::snmpget(".1.3.6.1.2.1.25.3.3.1.2.".$regs[1]);
						$cpu[]=array('index'=>$regs[1],'load'=>formatInteger($load));
					}
				}
			}
		}
		else
		{
			$load =parent::snmpget(".1.3.6.1.2.1.25.3.3.1.2.".$index);
			$cpu[]=array('index'=>$index,'load'=>formatInteger($load));
		}
		return $cpu;
	}

	public function getStorage(){
		$columns=array(3=>"hrStorageDescr","hrStorageAllocationUnits","hrStorageSize","hrStorageUsed");
		if(empty($this->storage)) $this->storage=$this->snmpToArray("1.3.6.1.2.1.25.2.3.1",$columns);
		return $this->storage;
	}
	
	public function getInterfaces(){
		$columns=array(1=>"ifIndex","ifDescr","ifType","ifMtu",6=>"ifPhysAddress","ifAdminStatus","ifOperStatus");
		if(empty($this->interfaces)) 
		{
			$this->interfaces=$this->snmpToArray("1.3.6.1.2.1.2.2.1",$columns);
			try
			{
				foreach($this->interfaces as $key => $line)
				{
					$this->interfaces[$key]['ifName'] = parent::snmpget("1.3.6.1.2.1.31.1.1.1.1.".$this->interfaces[$key]['ifIndex']);
				}
			}catch(Exception $e){}
		}
		
		return $this->interfaces;
	}
	
	public function getIPv4Adresses(){
		$columns=array(1=>"ipAdEntAddr","ipAdEntIfIndex","ipAdEntNetMask");
		if(empty($this->ipv4addr)) $this->ipv4addr=$this->snmpToArray("1.3.6.1.2.1.4.20.1",$columns);
		return $this->ipv4addr;
	}
	
	public function getHostSystemInfo(){
		$columns=array(1=>"hrSystemUptime","hrSystemDate",4=>"hrSystemInitialLoadParameters","hrSystemNumUsers","hrSystemProcesses","hrSystemMaxProcesses");
		if($this->hostSystemInfo==NULL) {
			$this->hostSystemInfo=$this->snmpToArray("1.3.6.1.2.1.25.1",$columns);
			$arr=explode(" ",$this->hostSystemInfo[0]["hrSystemDate"]);
			$year=hexdec($arr[0].$arr[1]);
			$month=hexdec($arr[2]);
			$day=hexdec($arr[3]);
			$hour=hexdec($arr[4]);
			$min=hexdec($arr[5]);
			$sec=hexdec($arr[6]);
			
			$this->hostSystemInfo[0]["hrSystemDate"]=sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year,$month,$day,$hour,$min,$sec);
			
			preg_match("/\((\d*)\)/",$this->hostSystemInfo[0]["hrSystemUptime"] ,$matches);
			$this->hostSystemInfo[0]["hrSystemUptime"]=$matches[1];
			
		}
		return $this->hostSystemInfo;
	}
	
	public function getProcesses(){
		$columns=array(1=>"hrSWRunIndex","hrSWRunName",4=>"hrSWRunPath","hrSWRunParameters",7=>"hrSWRunStatus");
		if($this->processes==NULL) {
			$this->processes=$this->snmpToArray("1.3.6.1.2.1.25.4.2.1",$columns);
			//print_r($this->processes);
			$walk=$this->snmpwalkoid("1.3.6.1.2.1.25.5.1.1");
			for($i=0;$i<count($this->processes);$i++){
				$this->processes[$i]["hrSWRunPerfCPU"]=$this->parse_raw_data($walk["iso.3.6.1.2.1.25.5.1.1.1.".$this->processes[$i]["hrSWRunIndex"]]);
				$this->processes[$i]["hrSWRunPerfMem"]=$this->parse_raw_data($walk["iso.3.6.1.2.1.25.5.1.1.2.".$this->processes[$i]["hrSWRunIndex"]]);
			}
		}
		return $this->processes;
	}
	
	public function getIPStats(){
		$columns=array(
				4=>"ipInHdrErrors",
				"ipInAddrErrors",
				
				"ipForwDatagrams",
				
				"ipInUnknownProtos",
				"ipInDiscards",
				"ipInDelivers",
				
				"ipOutRequests",
				"ipOutDiscards",
				"ipOutNoRoutes",
				
				"ipReasmTimeout",
				"ipReasmReqds",
				"ipReasmOKs",
				"ipReasmFails",
				
				"ipFragOKs",
				"ipFragFails",
				"ipFragCreates",
				
				23=> "ipRoutingDiscards",
				);
		if($this->ipStats==NULL) $this->ipStats=$this->snmpToArray("1.3.6.1.2.1.4",$columns);
		return $this->ipStats;
	}
	
	public function getUDPStats(){
		$columns=array(1=>"udpInDatagrams","udpNoPorts","udpInErrors","udpOutDatagrams");
		if($this->udpStats==NULL) $this->udpStats=$this->snmpToArray("1.3.6.1.2.1.7",$columns);
		return $this->udpStats;
	}
	
	public function getUDPTable(){
		$columns=array(1=>"udpLocalAddress","udpLocalPort");
		if($this->udpTable==NULL) $this->udpTable=$this->snmpToArray("1.3.6.1.2.1.7.5.1",$columns);
		return $this->udpTable;
	}
	
	public function getTCPStats(){
		$columns=array(
				2=>"tcpRtoMin",
				"tcpRtoMax",
				"tcpMaxConn",
				"tcpActiveOpens",
				"tcpPassiveOpens",
				"tcpAttemptFails",
				"tcpEstabResets",
				"tcpCurrEstab",
				"tcpInSegs",
				"tcpOutSegs",
				"tcpRetransSegs",
				
				14=>"tcpInErrs",
				"tcpOutRsts",
		);
		if($this->tcpStats==NULL) $this->tcpStats=$this->snmpToArray("1.3.6.1.2.1.6",$columns);
		return $this->tcpStats;
	}
	
	public function getTCPTable(){
		$columns=array(1=>"tcpConnState","tcpConnLocalAddress","tcpConnLocalPort","tcpConnRemAddress","tcpConnRemPort");
		if($this->tcpTable==NULL) $this->tcpTable=$this->snmpToArray("1.3.6.1.2.1.6.13.1",$columns);
		return $this->tcpTable;
	}
	
	private function parse_raw_data($data){
		/*$ret;
			$raw_data_arr=explode(":",($data));
			if(isset($raw_data_arr[1])) {
				$ret=$raw_data_arr[1];
				
				$ret= trim($ret);
				if($ret[0]=='"' && $ret[strlen($ret)-1]=='"'){
					$ret=explode('"',$ret)[1];
				}
				return trim($ret);
			}*/

		// suppression par exemple des INTEGER, COUNTER,etc du début
		//ainsi que les " dans le cas des chaines de caractères 

		$tmp = preg_replace('#^([a-zA-Z0-9\-]*\:\s+)#','',$data);
		$tmp = trim($tmp,"\"\t\n\r\0\x0B");

		return $tmp;
	}
	
	private function snmpToArray($table_oid,$columns){
		//Construction de la regex
		$oid_nodes=explode(".",$table_oid);
		array_shift ( $oid_nodes ); 
		$regex='#((iso|1)\.'.implode("\.",$oid_nodes).')\.1\.(.*)#';
		
		//Récupération des données et construction du tableau
		$walk=$this->snmpwalkoid($table_oid);
		$ret=array();
		foreach($walk as $oid => $data)
		{
			//echo $data.'<br />';
			if(!preg_match($regex,$oid,$matches)) break;
			$line=array();
			foreach($columns as $index => $column){
				$line[$column]="";
				if(isset($walk["$matches[1].$index.$matches[3]"])){
					$line[$column]=$this->parse_raw_data($walk["$matches[1].$index.$matches[3]"]);
				}
			}
			array_push($ret, $line);
		}
		return $ret;	
	}
}
?>
