<?php
/**
 * @name  Model_server
 * @description Classe héritant de "Model_esia" gèrent tout les accès à la base de donnée du plugin Server
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 08/11/2012
 */
include_once SNMP_CLASS;
include_once DEVICE_CLASS;
include_once MVCMODEL;
include_once './libs/RemoteServer.class.php';
if(!class_exists('Server'))
{
	include_once './libs/Server.class.php';

}

class Model_server extends Model_esia
{
	/*@NAME: getInstanceServer
	 *@IN: $dbconnection un objet PDO
	* 	$server nom ou ID du serveur
	*	$manufacturer (optionnelle) pas encore implémenté
	*@OUT: un objet Server
	*@TREAT: Instance un objet server
	*@EXCEPTION: 	EsiaDbException  si une erreur avec la BD survient
	*/
	public static function InstanceServer($dbconnection,$server,$manufacturer=null)
	{
		try
		{
			$node = Model_esia::InstanceNode($dbconnection,$server);
			$srv = New Server($node);
			if(@get_class($node)=='RemoteDevice')
			{
				$srv = New RemoteServer($node,$node->getGESA() );
			}
			
			return $srv;
		}
		catch(EsiaDbException $e)
		{
			throw new ModelServerException('Errors in instanciation Server\'s object'.$server,1,$e);
		}
	}
	/*@NAME: getHealthPlugin
	 *@IN: $dbconnection un objet PDO
	*@OUT: un tableau associatif
	*	 array('SRV_CPU_LOAD' => array(nom des plugins se rapportant à la charge CPU),
			* 	 	'SRV_HDD_LOAD' => array(nom des plugins se rapportant au disque dur,
					'SRV_RAM_LOAD' => array(nom des plugins se rapportant à la RAM),
					*@TREAT: effectue une requête sur la base de donnée (table: esia_configuration clé SRV_*) pour récupérer la liste des plugins "health"
					*@EXCEPTION: 	/
					*/
	public static function getHealthPlugin($dbconnection)
	{
		$query= $dbconnection->prepare("SELECT config_key, config_value
				FROM esia_configuration
				WHERE config_key LIKE 'SRV_%';");
		$result = $query->execute();
		if($result===false)
		{
			$err = $query->errorInfo();
			throw new ModelServerException($err[2]);
		}
		$plugin_srv = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$plugin_srv[$row->config_key]=explode(":",$row->config_value);
		}
		return $plugin_srv;
	}
}
class ModelServerException extends EsiaDbException{}
?>
